"""
Exception raised when the requested resource(route, file, database record) is no longer available and will not be
available again
"""


class GoneException(BaseException):
    """
    GoneException
    """
    pass
