"""
Exception raised when the theres a conflict in the request, such as an edit conflict between multiple simultaneous
updates
"""


class ConflictException(BaseException):
    """
    ConflictException
    """
    pass
