"""
Exception raised when the App tries to comunicate with third-party online services and this comunications fails
"""


class ServiceUnavailableException(BaseException):
    """
    ServiceUnavailableException
    """
    pass
