"""
Exception raised when a body request could be parsed normally, but it's contents doesn't satisfies the specified
values by the app
"""


class UnacceptableInputException(BaseException):
    """
    UnacceptableInputException
    """
    pass
